/* Convert binary file into Intel HEX format                                 */
/* Compile this using the Compact model to get big file buffers              */

/*---------------------------------------------------------------------------*/
/* Microsoft headers                                                         */

#include <conio.h>
#include <dos.h>
#include <fcntl.h>
#include <io.h>
#include <memory.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys\types.h>
#include <sys\stat.h>

/*---------------------------------------------------------------------------*/
/* Blaise tools headers                                                      */

#include <bfiles.h>

/*---------------------------------------------------------------------------*/
/* Constants and suchlike                                                    */

#define BYTESPERLINE 32                /* number of data bytes in HEX lines  */

/*---------------------------------------------------------------------------*/
/* The Main Event                                                            */

int main(int argc,char *argv[]) {

   int  hBINfile;                      /* input binary file handle           */
   char BINspec[MAX_FLEN];             /*  ... and name                      */
   int BINfname;                       /* offset of file name within BINspec */
   struct stat BINinfo;                /* information about file             */
   unsigned char far *pBINdata;        /* internal buffer for binary data    */

   FILE *pHEXfile;                     /* output hex file stream             */
   char HEXspec[MAX_FLEN];             /*  ... and name                      */
   unsigned int HEXbase;               /* starting address for output        */

   int linectr;
   unsigned int linebase;
   int nlines;
   int nleft;
   unsigned char *pdata;
   int checksum;

   char *errmsg;                       /* ending error message               */
   int retcode = 1;                    /*  and exit code (assume failure)    */

   printf("Binary file HEXifier\n\n");
   printf("Nisley Micro Consulting\n\n");

   if (argc == 1) {
      errmsg = "Syntax is:\n"
            "  HEXIFY filespec [base]\n\n"
            "where filespec names the input binary file\n"
            "      base sets the first output address (hex value)\n"
            "If you omit the input file extension, we assume .BIN\n"
            "The default base address is 0000 hex\n"
            "The output file will have a .HEX extension\n";
     goto bailout;
   }

/*--- if there are enough arguments, figure out the base address             */

   if (argc == 3) {
      HEXbase = (unsigned int) strtol(argv[2],&pdata,16);
   }
   else {
      HEXbase = 0;
   }

/*--- get the input file straightened out and grab it all in one shot        */

   if (flnorm(argv[1],BINspec,&BINfname)) {
      printf("Invalid binary file spec: %s\n",argv[1]);
      goto bailout;
   }

   if (NULL == strchr(BINspec,'.')) {
      printf("Assuming .BIN extension\n");
      strcat(BINspec,".BIN");
   }

   printf("Binary input file is %s\n",BINspec);

   if (-1 == (hBINfile = open(BINspec,O_BINARY|O_RDONLY))) {
      errmsg = "Can't open binary file for reading";
      goto bailout;
   }

   if (fstat(hBINfile,&BINinfo)) {
      errmsg = "Can't get info about input file";
      goto bailout;
   }

   if (BINinfo.st_size >= 0xffff) {
      errmsg = "Input file is too big!";
      goto bailout;
   }

   if (NULL == (pBINdata = malloc((unsigned)BINinfo.st_size))) {
      errmsg = "Not enough memory for file buffer";
      goto bailout;
   }

   if ((unsigned)BINinfo.st_size !=
         read(hBINfile,pBINdata,(unsigned)BINinfo.st_size)) {
      errmsg = "Problem reading input file";
      goto bailout;
   }

   close(hBINfile);


/*--- create the output file name and open it up                             */

   strcpy(HEXspec,BINspec);

   strcpy(strrchr(HEXspec,'.'),".HEX");

   printf("HEX   output file is %s\n",HEXspec);

   if (NULL == (pHEXfile = fopen(HEXspec,"w"))) {
      free(pBINdata);
      errmsg = "Can't open HEX file for writing";
      goto bailout;
   }

   if (HEXbase) {
      printf("  starting address = %4.4X\n",HEXbase);
   }


/*--- dump all the whole lines                                               */

   linebase = HEXbase;
   linectr = 1;
   pdata = pBINdata;

   for (nlines = ((unsigned)BINinfo.st_size) / BYTESPERLINE;
        nlines > 0;
        (nlines--),(linectr++),(linebase+=BYTESPERLINE)){
      fprintf(pHEXfile,":%2.2X%2.2X%2.2X00",
                BYTESPERLINE,(linebase >> 8),(linebase & 0x00ff));
      checksum = (linebase >> 8) + (linebase & 0x00ff) + BYTESPERLINE;
      for (nleft = BYTESPERLINE;
           nleft > 0;
           nleft--) {
         checksum += *pdata;
         fprintf(pHEXfile,"%2.2X",*pdata++);
      }
      fprintf(pHEXfile,"%2.2X\n",((-checksum) & 0x00ff));
   }

/*--- and the partial line at the end                                        */

   if (nleft = ((unsigned)BINinfo.st_size) % BYTESPERLINE) {
      fprintf(pHEXfile,":%2.2X%2.2X%2.2X00",
                nleft,(linebase >> 8),(linebase & 0x00ff));
      checksum = (linebase >> 8) + (linebase & 0x00ff) + nleft;
      linebase += nleft;
      for (;
           nleft > 0;
           nleft--) {
         checksum += *pdata;
         fprintf(pHEXfile,"%2.2X",*pdata++);
      }
      fprintf(pHEXfile,"%2.2X\n",((-checksum) & 0x00ff));
      linectr++;
   }

/*--- the final end record is always the same                                */

   fprintf(pHEXfile,":00000001FF\n");

   if (fclose(pHEXfile)) {
      errmsg = "Problem closing HEX file!";
      goto bailout;
   }

   free(pBINdata);
   retcode = 0;


/*--- common exit point                                                      */

bailout :

   if (retcode) {
      printf("%s\n",errmsg);
   }

   return retcode;

}
