/* Micro-C program to demonstrate I/O timing                                 */
/* You need a scope to watch the bus signals during the loops                */

#include <8086io.h>
#include "8086base.h"

#define BOARD_BASE   0x0300            /* board starting address             */

#define LED_OFFSET   0x001E            /* offset of LED latches              */
#define SW_OFFSET    0x001E            /* offset of switch inputs            */

#define LPT_BASE     0x0378            /* parallel printer port address      */

/*---------------------------------------------------------------------------*/
/* Delay for a specified number of BIOS timer ticks                          */
/* Returns current BIOS tick counter                                         */

WORD WaitTicks(Delay)
WORD Delay;
{

WORD CurrentTick;
WORD StopTick;

   StopTick = peekw(0x0040,0x006C) + Delay;

   do {
      CurrentTick = peekw(0x0040,0x006C);
   } while (CurrentTick != StopTick);

   return CurrentTick;

}

/*---------------------------------------------------------------------------*/
/* The main event                                                            */

char Option;
WORD PortAddr;

main() {

   serinit(9600,1);
   enable();

   putstr("I/O Timing Tests\n");
   putstr("Embedded PC's ISA Bus Chapter 3 -- Ed Nisley\n");


   while (TRUE) {

      putstr("Bus timing routines:\n");
      printf(" 1 - 8-bit I/O to parallel port at %04x\n",LPT_BASE);
      printf(" 2 - 16-bit I/O to LED port at %04x, no recovery time\n",
               BOARD_BASE+LED_OFFSET);
      printf(" 3 - 16-bit I/O to LED port at %04x, short jump\n",
               BOARD_BASE+LED_OFFSET);
      putstr("Select option: ");

      Option = getch();
      putch(Option);
      putstr("    (press any key to exit test loops)\n\n");

      switch (Option) {
      case '1' :
         putstr("Sync on LPT bit 0, watch -IOW signal\n");
         PortAddr = LPT_BASE;
         while (!chkch()) {
            disable();
            outp(PortAddr,0x01);                   /* rising edge            */
            outp(PortAddr,0x00);                   /* falling edge           */
            asm {
               MOV   DX,PortAddr
               MOV   AL,#$01
               OUT   DX,AL
               MOV   AL,#$00
               OUT   DX,AL
            }
            enable();
         }
         break;
      case '2' :
         putstr("Sync on LPT bit 0, watch LED bit 0, -BIOW, -IO Addr Match\n");
         PortAddr = BOARD_BASE + LED_OFFSET;
         while (!chkch()) {
            disable();
            outp(LPT_BASE,0x01);                   /* scope sync             */
            outp(LPT_BASE,0x00);
            outpw(PortAddr,0xFF);                  /* rising edge            */
            outpw(PortAddr,0xFE);                  /* falling edge           */
            asm {
               MOV   DX,PortAddr
               MOV   AX,#$0001
               OUT   DX,AX
               MOV   AL,#$0000
               OUT   DX,AX
            }
            enable();
         }
         break;
      case '3' :
         putstr("Sync on LPT bit 0, watch LED bit 0, -BIOW, -IO Addr Match\n");
         PortAddr = BOARD_BASE + LED_OFFSET;
         while (!chkch()) {
            disable();
            outp(LPT_BASE,0x01);                   /* scope sync             */
            outp(LPT_BASE,0x00);
            outpw(PortAddr,0xFF);                  /* rising edge            */
            outpw(PortAddr,0xFE);                  /* falling edge           */
            asm {
               MOV   DX,PortAddr
               MOV   AX,#$0001
               OUT   DX,AX
               JMP   <?punt
?punt          MOV   AX,#$0000
               OUT   DX,AX
            }
            enable();
         }
         break;
      default :
         putstr("*** Choose one of the menu options\n\n");
      }

   }


}
