; Simple routine to demonstrate BIOS extensions
; Must be loaded into EEPROM/RAM with the LOADEXT.ASM boot loader
;  (using firmware write enable or with the write-enable jumper in)
; After it's loaded, remove the diskette and reboot to see the LEDs blink...
;  ... not too inspiring, but a great confidence builder!
;  low order DIP switch must be ON to enable the extension
;   set the switch OFF to reboot =without= this thing taking over!

; Embedded PC's ISA Bus Chapter 7
; Assemble with Borland TASM


              IDEAL
              MODEL     TINY

;--------------
; Set up a few constants

LED_ADDR      EQU       0031Eh              ; LED digits
SW_ADDR       EQU       0031Eh              ; DIP switch inputs

;-------------
; This does not start at offset 0100, so be careful about data accesses!
; Actual execution is at absolute address C800:0004...
; The first three bytes are the signature and length

              CODESEG
              STARTUPCODE

              DB        055h                ; signature
              DB        0AAh

              DB        1                   ; length in units of 512 bytes

MainEntry:
              JMP       SHORT Booter        ; force two-byte jump
              DB        000h                ; zero checksum until loaded

Booter:
              MOV       DX,SW_ADDR          ; should we run?
              IN        AX,DX
              MOV       DX,LED_ADDR         ; set up for display & switches

              TEST      AL,01h              ; low switch ON?
              JZ        Onward              ; zero says yes, so stay here

              MOV       AX,08080h           ; show we were here
              NOT       AX
              OUT       DX,AX               ;  ... just decimal points!

              RETF                          ; and return to BIOS!
Onward:

ReShow:
              MOV       AX,0FFFFh           ; all LEDs go off
              OUT       DX,AX

              MOV       CX,0
Wait1:        LOOP      Wait1

              MOV       AX,00000h           ; all LEDs go on
              OUT       DX,AX

              MOV       CX,0
Wait2:        LOOP      Wait2

              JMP       ReShow              ; continue forever

              END

