* Firmware Development Card routines
*
*
*-------------
* Set up BIOS real-time alarm clock to delay interval in microseconds
*  WORD SetAlarm(DelayHi,DelayLow,pbAlarm)
*  WORD DelayHi;
*  WORD DelayLow;
*  BYTE *pbAlarm;
* Returns 0 if OK, error code if not (also sets *pbAlarm to return code)
* High-order bit of *pbAlarm is set when delay time expires
* Remember to split the microsecond count manually... 1 sec = 1000000 us
*
SetAlarm      MOV       BX,SP               aim at the stack
              MOV       CX,6[BX]            get delay high word
              MOV       DX,4[BX]              ... low word
              MOV       BX,2[BX]            get pointer to alarm byte
              MOV       AX,DS                 ... set up segment
              MOV       ES,AX
              MOV       AX,#$8300           Set Event Wait Interval function
              INT       $15
              JC        ?1                  C set on error
              XOR       AL,AL                 ... clear return flag
?1            MOV       BX,SP               set up for return code
              MOV       BX,2[BX]
              MOV       ES:[BX],AL
              XOR       AH,AH               clear high byte of return value
*
              RET
*
*
*
*-------------
* Cancel pending BIOS real-time clock alarm
*  WORD CancelAlarm(DelayHi,DelayLow,pbAlarm)
*  WORD DelayHi;
*  WORD DelayLow;
*  BYTE *pbAlarm;
* Returns 0 if OK, 1 if error
*
CancelAlarm   MOV       BX,SP               aim at the stack
              MOV       CX,6[BX]            get delay high word
              MOV       DX,4[BX]              ... low word
              MOV       BX,2[BX]            get pointer to alarm byte
              MOV       AX,DS                 ... set up segment
              MOV       ES,AX
              MOV       AX,#$8301           Cancel Event Wait function
              INT       $15
              MOV       AX,#0               set up return code
              ADC       AL,#0               C set causes increment on error
*
              RET
*
*
*
*-------------
* BIOS delay interval in microseconds
*  WORD BIOSWait(DelayHi,DelayLow)
*  WORD DelayHi;
*  WORD DelayLow;
* Returns 0 if OK, error code if not
* Remember to split the microsecond count manually... 1 sec = 1000000 us
*
BIOSWait      MOV       BX,SP               aim at the stack
              MOV       CX,4[BX]            get delay high word
              MOV       DX,2[BX]              ... low word
              MOV       AX,#$8600           Set Event Wait Interval function
              INT       $15
              JC        ?2                  C set on error
              XOR       AH,AH                 ... clear return flag
?2            MOV       AL,AH               set up for return code
              XOR       AH,AH               clear high byte of return value
*
              RET
*
*
*
*-------------
* Convert byte into two seven-segment digits
*  WORD ByteToSegs(Value)
*  int Value;
* Returns 16-bit value with both decimal points turned off
*  ... remember to invert the value before writing it out
*
*--- Table of segment values
?3            DB        $7E
              DB        $30
              DB        $6D
              DB        $79
              DB        $33
              DB        $5B
              DB        $5F
              DB        $70
              DB        $7F
              DB        $7B
              DB        $77
              DB        $1F
              DB        $4E
              DB        $3D
              DB        $4F
              DB        $47
*
*
ByteToSegs    MOV       DX,DS               save data seg
              MOV       CX,CS               set up pointer to code segment
              MOV       DS,CX
*
              MOV       BX,SP               aim at the stack
              MOV       AX,2[BX]            get byte value
              XOR       AH,AH               clear high byte
              MOV       CX,#4               set up shift count
              ROR       AX,CL               split nibbles
              MOV       BX,#?3              aim at translation table
              XLAT                          convert high nibble
              XCHG      AH,AL               recover low nibble
              ROR       AL,CL               line it up
              XLAT                          conver low nibble
*
              MOV       DS,DX               restore data seg
              RET
*
*-------------
* Return segment address of our code
*  WORD GetCodeSeg();
* This is useful for setting interrupt vectors...
GetCodeSeg    MOV       AX,CS
              RET
*
*
*-------------
* Return segment address of our data
*  WORD GetDataSeg();
* This is useful for setting interrupt vectors...
GetDataSeg    MOV       AX,DS
              RET
*
*
*-------------
* Get interrupt vector
*  GetVector(IntNum,&pSegment,&pOffset)
*  int IntNum;
*  WORD *pSegment;
*  WORD *pOffset;
* Because Micro-C does not handle long ints, we use two pointers
* Turning off interrupts in this routine is a flourish
*
GetVector     MOV       BX,SP               aim at the stack
              MOV       SI,6[BX]            get int number
              AND       SI,#$00FF           make sure it is a byte
              ADD       SI,SI               convert to offset
              ADD       SI,SI
              XOR       AX,AX                and load segment
              MOV       ES,AX
              MOV       DI,2[BX]            transfer offset
              PUSHF                         save flags
              CLI
              MOV       AX,ES:[SI]
              MOV       [DI],AX
              ADD       SI,#2
              MOV       DI,4[BX]            transfer segment
              MOV       AX,ES:[SI]
              POPF                          restore interrupt flag
              MOV       [DI],AX
              RET
*
*
*-------------
* Set interrupt vector
*  SetVector(IntNum,Segment,Offset)
*  int IntNum;
*  WORD Segment;
*  WORD Offset;
*
SetVector     MOV       BX,SP               aim at the stack
              MOV       DI,6[BX]            get int number
              AND       DI,#$00FF           make sure it is a byte
              ADD       DI,DI               convert to offset
              ADD       DI,DI
              XOR       AX,AX                and load segment
              MOV       ES,AX
              MOV       AX,2[BX]            transfer offset
              PUSHF                         save flags
              CLI                           prevent interruptions
              MOV       ES:[DI],AX
              ADD       DI,#2
              MOV       AX,4[BX]            transfer segment
              MOV       ES:[DI],AX
              POPF                          restore interrupt flag
              RET
*
*
*-------------
* Load 8254 timer channel
*  LoadTimer(Chan,Ctl,Time,Base)
*  int Chan;
*  WORD Ctl;
*  WORD Time;
*  WORD Base;
* Forces RW = 11 to ensure two-byte read/write operations
* Does not check channel for validity...
*
LoadTimer     MOV       BX,SP               aim at the stack
              MOV       AX,8[BX]            get channel
              MOV       CX,#6               align to high bits
              SHL       AL,CL
              OR        AX,6[BX]            combine with control word
              OR        AL,#$30             force RW = 11
              MOV       DX,2[BX]            set up output address
              ADD       DX,#6               aim at control register
              OUT       DX,AL               write as a byte
              MOV       DX,2[BX]            aim at timer channel
              ADD       DX,8[BX]
              ADD       DX,8[BX]
              MOV       AX,4[BX]            fetch timer value
              OUT       DX,AL               write LSB as byte
              XCHG      AH,AL
              OUT       DX,AL                ... MSB
              RET
*
*
*-------------
* Read 8254 timer channel
*  WORD ReadTimer(Chan,Base)
*  int Chan;
*  WORD Base;
ReadTimer     MOV       BX,SP               aim at the stack
              MOV       AX,4[BX]            get channel
              MOV       CX,#6               align to high bits
              SHL       AL,CL                 (which makes a latch command)
              MOV       DX,2[BX]            set up output address
              ADD       DX,#6               aim at control register
              OUT       DX,AL               latch the channel
              MOV       DX,2[BX]            aim at timer channel
              ADD       DX,4[BX]
              ADD       DX,4[BX]
              IN        AL,DX               recover LSB
              XCHG      AH,AL
              IN        AL,DX                ... MSB
              XCHG      AH,AL               swap bytes for return
              RET
*
*
