/* Firmware Furnace Embedded '386SX Projects...                              */
/* Standard include file for 8086 programs                                   */

/*---------------------------------------------------------------------------*/
/* Macro wrapper for Micro-C hardware interrupt handlers (in TINY model)     */
/* To convert a function to an interrupt handler...                          */
/*  start with a standard function declaration:                              */
/*    StdFunc() {                                                            */
/*  add this macro thusly:                                                   */
/*    INTERRUPT(IntStdFunc) StdFunc() {                                      */
/*  aim the interrupt vector at the macro wrapper:                           */
/*    SetVector(0x30,INT_ENTRY(StdFunc());                                   */
/*  turn on the interrupts by twiddling the hardware                         */
/*  you may also invoke the function directly from the C code:               */
/*    StdFunc();                                                             */
/* Thanks to Dave Dunfield for suggesting this trick                         */

#define INT_PROLOGUE 30                /* size of prologue code              */

#define INT_ENTRY(fn)   (&fn-INT_PROLOGUE)         /* start of prologue      */

#define _SPACE_
#define INTERRUPT(fn) asm { \
fn PUSH  AX \
   PUSH  BX \
   PUSH  CX \
   PUSH  DX \
   PUSH  SI \
   PUSH  DI \
   PUSH  ES \
   PUSH  DS \
   PUSH  CS \
   POP   DS \
   MOV   AX,?temp \
   PUSH  AX \
   CALL  fn+INT_PROLOGUE \
   POP   AX \
   MOV   _SPACE_?temp,AX \
   POP   DS \
   POP   ES \
   POP   DI \
   POP   SI \
   POP   DX \
   POP   CX \
   POP   BX \
   POP   AX \
   IRET \
}

#undef _SPACE_


/*---------------------------------------------------------------------------*/
/* Galactic standard PC addresses and so forth                               */

#define I8259A    0x20                 /* primary interrupt controller       */
#define I8259B    0xA0                 /* secondary interrupt controller     */

#define I8259A_VECTOR   0x08           /* Primary controller base vector     */
#define I8259B_VECTOR   0x70           /*  ... ditto, secondary controller   */

#define NS_EOI    0x20                 /* non-specific End-Of-Interrupt      */

#define SYS_CTLS  0x61                 /* AT system control bit port         */
#define SYS_CTLS_A $61

#define KEY_DATA     0x60              /* keyboard controller data/status    */
#define KEY_DATA_A   $60

#define KEY_CMD      0x64              /* keyboard controller command port   */
#define KEY_CMD_A    $64

#define RTC_ADDR     0x70              /* Real-time clock address port       */
#define RTC_ADDR_A   $70               /*  MSB = 1 disables NMI!             */

#define RTC_DATA     0x71              /* Real-time clock data port          */
#define RTC_DATA_A   $71


/*---------------------------------------------------------------------------*/
/* Handy stuff                                                               */

#define BYTE unsigned char
#define WORD unsigned int

#define TRUE 1
#define FALSE !TRUE


#define HI(x) (x >> 8)                 /* move high byte to low byte         */
#define LO(x) (x & 0x00FF)             /* isolate low byte                   */


/*--- useful character definitions                                           */
/*    these are in decimal so as to be useful in either C or assembler       */

#ifndef CANCEL

#define CANCEL    3
#define BELL      7
#define BS        8
#define TAB       9
#define CR        13
#define LF        10
#define FORMFEED  12
#define ESC       27
#define BLANK     ' '
#define DEL       127

#define XON       17
#define XOFF      19

#endif

#ifndef EOF
#define EOF -1
#endif

/*---------------------------------------------------------------------------*/
/* Assembler macros                                                          */

#define LOW(x) x
#define HIGH(x) =x

#define ASWORD(x) >x
#define ASBYTE(x) <x
