* Micro-C ROM memory model code modified for BIOS extension use
*
* Replace the top part of the normal Micro-C ROM model code in 8086RLPR.ASM
*  with this file...
*  zap everything from the first non-comment line down to the "exit" loop
*
* Then move these two lines to the bottom of the startup code, just after
*  the "$DD:?temp 2" line... and un-comment them, of course
***$DD:?BIOS_SP 2
***$DD:?BIOS_SS 2
*-------------
*
*--- Useful constants
*
STK_TOP       EQU       $7FFE               one word below boundary
*
LED_ADDR      EQU       $031E               LED output port
STAT_ADDR     EQU       $031C               status input port
PUSHBUTTON    EQU       $0200               pushbutton bit location
*
CTLS_ADDR     EQU       $031C               FDB control output port
NV_WENABLE    EQU       $0200               1 = enable writes into RAM
*
NV_SEGMENT    EQU       $C800               non-volatile memory base
*
*
*-------------
* BIOS Extension header
*
              ORG       $0100               standard COM offset
*
              DB        $55                 0000 signature
              DB        $AA                 0001 ... second byte
              DB        6                   0002 code + init data size
              JMP       <BootEntry          0003 jump around checksum
              DB        $00                 0005 ... which goes here
*
*-------------
* Define "read-only" variables
* We store these during the boot sequence and use them during execution
* Put them at the end of the battery-backed RAM to avoid the checksum
* The offsets are relative to the physical memory segment!
*  add 0100 to get them relative to the extension code segment
*
STACKTOPOFF   EQU       $7F00
DATASEGMENT   EQU       $7F02
OLDINT19OFF   EQU       $7F04
OLDINT19SEG   EQU       $7F06
OLDINT18OFF   EQU       $7F08
OLDINT18SEG   EQU       $7F0A

*-------------
* Now proceed with boot code
*
*--- if pushbutton is down, exit without doing much
*
BootEntry     EQU       *
              MOV       DX,#STAT_ADDR
              IN        AX,DX
              TEST      AX,#PUSHBUTTON
              JNZ       <Continue           nonzero means not pushed
*
              MOV       AX,#~$0101          show -- to track our path
              MOV       DX,#LED_ADDR
              OUT       DX,AX
*
              RETF                          ; return to normal BIOS boot
*
*--- pushbutton is up, so it is OK to continue
*    adjust CS so our offsets are correct
*
Continue      EQU       *
              MOV       AX,#~$0100          show single - here
              MOV       DX,#LED_ADDR
              OUT       DX,AX
*
              MOV       AX,CS
              SUB       AX,#$0010           add 0100 to offsets!
              PUSH      AX                  simulate a FAR CALL
              MOV       AX,#LdRegs
              PUSH      AX
              RETF                          ;  ... to load the new values
LdRegs        EQU       *
*
*--- locate the data segment at the top 32K of contiguous RAM
*    the BIOS RAM size value at 40:13 has units of 1K bytes
*
              MOV       AX,#$057E           show r0 on LEDs to mark entry
              CALL      ShowBits
*
              PUSH      DS                  save BIOS seg regs
              PUSH      ES                   ... on the BIOS stack
*
              MOV       AX,#$0040           fetch memory size
              MOV       ES,AX
              MOV       BX,#$0013
              SUB       ES:[BX],#>$0020     reserve 32K (1K byte units)
              MOV       AX,ES:[BX]          convert RAM size to seg reg
              MOV       CL,#6                 (10 bits left, 4 bits right)
              SHL       AX,CL
              MOV       ES,AX               target seg for MOVSB and STOSB
*
*--- copy the initialized data to the data segment
*
              MOV       AX,#$0530           ; r1
              CALL      ShowBits
*
              MOV       AX,CS               aim at this segment
              MOV       DS,AX
              MOV       CX,#?Isize          pick up length
              MOV       SI,#?Ivars          start of init data in code seg
              MOV       DI,#0               copied to start of data seg
              JCXZ      ?nocopy             skip if zero (or 64 K) bytes long
              REP
              MOVSB                         from DS:SI to ES:DI
?nocopy       EQU       *
*
*--- clear the uninitialized data while we have the pointers at hand
*    length is the difference between ?heap and ?temp
*
              MOV       AX,#$056D           r2
              CALL      ShowBits
*
              MOV       CX,#?heap+1-?temp   length of uninit data + heap top
              MOV       AL,#0               get a zero (or a test value!)
              REP
              STOSB
*
*--- aim DS at the new data segment
*
              MOV       AX,ES
              MOV       DS,AX
*
*--- save the BIOS SS:SP registers
*     ... begin using our stack in the new data segment
*
              MOV       ?BIOS_SP,SP
              MOV       ?BIOS_SS,SS
*
              MOV       SS,AX               load stack pointer
              MOV       SP,#STK_TOP
*
*--- now save pointer to our data in the battery-backed RAM
*
              MOV       AX,#NV_WENABLE      allow writing into RAM
              MOV       DX,#CTLS_ADDR
              OUT       DX,AX
*
              MOV       AX,#NV_SEGMENT      aim at RAM segment
              MOV       ES,AX
              MOV       BX,#STACKTOPOFF      ... stack offset
              MOV       ES:[BX],SP           ... write offset
              MOV       ES:2[BX],DS          ... and data segment
*
              MOV       AX,#0               prohibit writing into RAM
              MOV       DX,#CTLS_ADDR
              OUT       DX,AX
*
              MOV       AX,DS
              MOV       ES,AX               restore ES
*
*--- now, for the main event...
*
              MOV       AX,#$0579           r3
              CALL      ShowBits
*
              MOV       <?heap,#0           set up empty heap
              CALL      main                Execute main program
*
*--- restore regs and return
*
              MOV       SS,?BIOS_SS         aim back at BIOS stack
              MOV       SP,?BIOS_SP
*
              POP       ES                  from BIOS stack
              POP       DS
*
              MOV       AX,#$0505           show rr to indicate the end
              CALL      ShowBits
*
              RETF                          ; and continue with boot
*
*-------------
* Write AX to the LED digits
*
ShowBits      EQU       *
              PUSH      DX
              MOV       DX,#LED_ADDR
              NOT       AX                  ; make 1 bits light LEDs
              OUT       DX,AX
              POP       DX
              RET
*
*--------------
* Rest of normal Micro-C startup code follows below...
*--------------
