/* Firmware Development Board                                                */
/* Port addresses and bit locations                                          */

#define BOARD_BASE   0x0300            /* board starting address             */
#define BOARD_BASE_A $0300
#define BOARD_SIZE   0x0020            /* address range on board             */

#define NV_SEGMENT   0xC800            /* non-volatile memory base           */
#define NV_SEGMENT_A $0C800
#define NV_SIZE      0x8000            /*  ... and size (32K = -32768!)      */

#define SYNC_ADDR    0x0378            /* LPT port addr for scope syncs &c   */
#define SYNC_ADDR_A  $378

#define I8254_BASE   (BOARD_BASE+0x0008)           /* base of timer regs     */
#define I8254_BASE_A (BOARD_BASE_A+$0008)
#define I8254_NS     139                           /* nanoseconds per count  */

/*--- miscellaneous status and control bits                                  */
/*    char LCD controller data I/O is through the low-order byte             */

#define CTLS_ADDR    (BOARD_BASE+0x001C)           /* write also sets        */
#define CTLS_ADDR_A  $031c                         /*  fast watchdog mode    */

#define WATCHDOG     0x0100                        /* watchdog toggle output */
#define WATCHDOG_A   $0100
#define NV_WENABLE   0x0200                        /* 1 = enable NV writes   */
#define NV_WENABLE_A $0200
#define DS2400OUT    0x0400                        /* DS2400 data output     */
#define DS2400OUT_A  $0400
#define LCD_E        0x1000                        /* LCD Enable signal      */
#define LCD_E_A      $1000
#define LCD_RW       0x2000                        /* LCD Rd/-Wr signal      */
#define LCD_RW_A     $2000
#define LCD_RS       0x4000                        /* LCD Register Select    */
#define LCD_RS_A     $4000
#define LCD_NODRV    0x8000                        /* +Disable data to LCD   */
#define LCD_NODRV_A  $8000

#define STAT_ADDR    (BOARD_BASE+0x001C)
#define STAT_ADDR_A  $031c

#define PWR_GOOD     0x0100                        /* 1 = power is still OK  */
#define PWR_GOOD_A   $0100
#define PUSHBUTTON   0x0200                        /* pushbutton input       */
#define PUSHBUTTON_A $0200
#define DS2400IN     0x0400                        /* DS2400 data output     */
#define DS2400IN_A   $0400

/*--- LEDs and DIP switch inputs                                             */

#define LED_ADDR     (BOARD_BASE+0x001E)           /* offset of LED latches  */
#define LED_ADDR_A   (BOARD_BASE_A+$001E)
#define SW_ADDR      (BOARD_BASE+0x001E)           /* offset of switch inputs*/
#define SW_ADDR_A    (BOARD_BASE_A+$001E)


/*---------------------------------------------------------------------------*/
/* Macro wrapper for Micro-C hardware interrupt handlers in SMALL/ROM model  */
/* We allocate a stack in a separate block of RAM pointed to by a variable   */
/*  stored in our battery-backed RAM... see text in Chapter 10 for details   */
/* You =must= have battery-backed RAM to make this work!                     */

/* These offsets are decimal to allow use in both assembler and C            */
/* See 8086RLXR.ASM for details                                              */

#define STACKTOPOFF   (32512+0)
#define DATASEGMENT   (32512+2)
#define OLDINT19OFF   (32512+4)
#define OLDINT19SEG   (32512+6)
#define OLDINT18OFF   (32512+8)
#define OLDINT18SEG   (32512+10)

#define INT_S_PROLOG 64                /* size of prologue code              */

#define INT_S_ENTRY(fn)   (&fn-INT_S_PROLOG)       /* start of prologue      */

#define _SPACE_
#define INT_SMALL(fn,stk) asm {                                      \
*--- save bystanders on incoming stack                               \
fn PUSH     AX                                                       \
   PUSH     BX                                                       \
   PUSH     CX                                                       \
   PUSH     DX                                                       \
   PUSH     SI                                                       \
   PUSH     DI                                                       \
   PUSH     ES                                                       \
   PUSH     DS                                                       \
*--- fetch ptr from battery-backed RAM to find DS/ES/SS and stack    \
   MOV      BX,#NV_SEGMENT_A                                         \
   MOV      ES,BX                                                    \
   MOV      BX,#STACKTOPOFF                                          \
   MOV      CX,ES:[BX]                                               \
   SUB      CX,#stk                                                  \
   MOV      AX,SS                                                    \
   MOV      BX,ES:2[BX]                                              \
   MOV      SS,BX                                                    \
   XCHG     SP,CX                                                    \
   PUSH     CX                                                       \
   PUSH     AX                                                       \
   MOV      DS,BX                                                    \
   MOV      ES,BX                                                    \
*--- save our compiler temp on our stack                             \
   MOV      AX,?temp                                                 \
   PUSH     AX                                                       \
*--- invoke the Micro-C handler                                      \
   CALL  fn+INT_S_PROLOG                                             \
*--- restore compiler temp from our stack                            \
   POP      AX                                                       \
   MOV      _SPACE_?temp,AX                                          \
*--- restore pointer to incoming stack                               \
   POP      AX                                                       \
   POP      CX                                                       \
   MOV      SS,AX                                                    \
   MOV      SP,CX                                                    \
   POP      DS                                                       \
   POP      ES                                                       \
   POP      DI                                                       \
   POP      SI                                                       \
   POP      DX                                                       \
   POP      CX                                                       \
   POP      BX                                                       \
   POP      AX                                                       \
*--- bypass Micro-C return                                           \
   IRET                                                              \
}


#undef _SPACE_


