/* Firmware Development Board                                                */
/* Port addresses and bit locations                                          */

#define BOARD_BASE   0x0300            /* board starting address             */
#define BOARD_BASE_A $0300
#define BOARD_SIZE   0x0020            /* address range on board             */

#define NV_SEGMENT   0xC800            /* non-volatile memory base           */
#define NV_SIZE      0x8000            /*  ... and size (32K = -32768!)      */

#define GLCD_SEGMENT 0xD000            /* graphics LCD buffer segment        */
#define GLCD_SIZE    0x8000            /*  ... and size in bytes             */

#define SYNC_ADDR    0x0378            /* LPT port addr for scope syncs &c   */
#define SYNC_ADDR_A  $378

#define I8254_BASE   (BOARD_BASE+0x0008)           /* base of timer regs     */
#define I8254_BASE_A (BOARD_BASE_A+$0008)
#define I8254_NS     139                           /* nanoseconds per count  */

/*--- miscellaneous status and control bits                                  */
/*    char LCD controller data I/O is through the low-order byte             */

#define CTLS_ADDR    (BOARD_BASE+0x001C)           /* write also sets        */
#define CTLS_ADDR_A  $031c                         /*  fast watchdog mode    */

#define WATCHDOG     0x0100                        /* watchdog toggle output */
#define WATCHDOG_A   $0100
#define NV_WENABLE   0x0200                        /* 1 = enable NV writes   */
#define NV_WENABLE_A $0200
#define DS2400OUT    0x0400                        /* DS2400 data output     */
#define DS2400OUT_A  $0400
#define LCD_E        0x1000                        /* LCD Enable signal      */
#define LCD_E_A      $1000
#define LCD_RW       0x2000                        /* LCD Rd/-Wr signal      */
#define LCD_RW_A     $2000
#define LCD_RS       0x4000                        /* LCD Register Select    */
#define LCD_RS_A     $4000
#define LCD_NODRV    0x8000                        /* +Disable data to LCD   */
#define LCD_NODRV_A  $8000

#define STAT_ADDR    (BOARD_BASE+0x001C)
#define STAT_ADDR_A  $031c

#define PWR_GOOD     0x0100                        /* 1 = power is still OK  */
#define PWR_GOOD_A   $0100
#define PUSHBUTTON   0x0200                        /* pushbutton input       */
#define PUSHBUTTON_A $0200
#define DS2400IN     0x0400                        /* DS2400 data output     */
#define DS2400IN_A   $0400

/*--- Graphics LCD interface                                                 */
/*    Note that some of the bits are minus-active and plus-disabling         */
/*    Use a bit-or (|) to combine the bits...                                */

#define GLCD_CTLS    (BOARD_BASE+0x01A)

#define GLCD_ENCTR     0x0001                      /* enable addr counters   */
#define GLCD_DISMUX    0x0002                      /* 1 disables output mux  */
#define GLCD_ENDISP    0x0004                      /* enable LCD (if used)   */

#define GLCD_BLSLOW    0x0050                      /* slowest blinking       */
#define GLCD_BLMED     0x0040                      /* moderate blinking      */
#define GLCD_BLFAST    0x0030                      /* fastest blinking       */
#define GLCD_BLZERO    0x0000                      /* output logic zero      */
#define GLCD_BLCLOCK   0x0060                      /* output LCD clock       */
#define GLCD_BLONE     0x0070                      /* output logic one       */

#define GLCD_OFF     (GLCD_DISMUX | GLCD_BLZERO)
#define GLCD_ON      (GLCD_ENCTR | GLCD_ENDISP)

/*--- LEDs and DIP switch inputs                                             */

#define LED_ADDR     (BOARD_BASE+0x001E)           /* offset of LED latches  */
#define LED_ADDR_A   (BOARD_BASE_A+$001E)
#define SW_ADDR      (BOARD_BASE+0x001E)           /* offset of switch inputs*/
#define SW_ADDR_A    (BOARD_BASE_A+$001E)


