; Graphic LCD Interface routines for 480x128 4-bit 960 ns panel
; This one is so weird that the standard boilerplate does not apply
;  ... remember we use only the odd bytes in the refresh buffer

              IDEAL
              P386N

              INCLUDE   "firmdev.inc"

              MODEL     SMALL,CPP

;-------------
; Define LCD parameters
; We must use = rather than EQU to get real numeric results!

LCDNAME       EQU       "Hitachi LM215 (960 ns, set jumpers!)"
NUMROWS       =         128
NUMCOLS       =         480
TOTALCLOCKS   =         30720
ROWCLOCKS     =         480                 ; every other byte in buffer!
COLMODULUS    =         NUMCOLS/2
ROWMODULUS    =         NUMROWS/2
FILLVALUE     =         000h

              EXTRN     C Load8254:PROC


;-------------
; Set up data values needed by the plotting routines
; The 386 division routines need in-storage divisors...

              DATASEG

              EXTRN     C pOldBuff:FAR PTR BYTE
              EXTRN     C pNewBuff:FAR PTR BYTE
              EXTRN     C pLCDBuff:FAR PTR BYTE

LCDName       DB        LCDNAME             ; fill in the name
              DB        0                   ;  ... and a terminator
RowClocks     DW        ROWCLOCKS           ; clocks, thus bytes, on each row
RowModulus    DW        ROWMODULUS          ; row addresses break here
ColModulus    DW        COLMODULUS          ; column addresses break here

RowStarts     DW        NUMROWS DUP (?)     ; row starting addresses

LCDCtlBits    DW        0                   ; copy of GLCD_CTLS output port

              CODESEG

              EXTRN     _Load8254

;-------------
; Enable & disable the LCD panel

              PUBLIC    C LCDEnable
              PROC      LCDEnable
              ARG       Enable : WORD

              MOV       AX,[LCDCtlBits]     ; pick up current controls

              TEST      [Enable],0001h      ; 1 = turn it on...
              JNZ       @@On

              OR        AX,GLCD_DISMUX      ; raise the MUX disable line
              JMP       SHORT @@Set
@@On:
              AND       AX,NOT GLCD_DISMUX  ; lower the disable line

@@Set:
              MOV       DX,GLCD_CTLS        ; tell the port
              OUT       DX,AX

              MOV       [LCDCtlBits],AX     ; and save for next time

              RET

              ENDP      LCDEnable


;-------------
; Initialize the Graphic LCD Interface hardware
; Also fills the buffer and returns some useful numbers
; This keeps the initialization code in here where it belongs...

              PUBLIC    C LCDInit
              PROC      LCDInit
              ARG       pNumRows : PTR WORD, pNumCols: PTR WORD, \
                        pNumBytes : WORD, ppName : PTR PTR BYTE
              USES      ES,DI

;--- disable the panel and set up the sync counters

              MOV       AL,GLCD_OFF         ; shut the interface down
              MOV       DX,GLCD_CTLS
              OUT       DX,AL

              CALL      Load8254 C,1,04h,ROWCLOCKS    ; use Mode 2
              CALL      Load8254 C,2,04h,TOTALCLOCKS  ;  ... for both values

              MOV       DI,[pNumRows]                 ; tell the caller
              MOV       [WORD DI],NUMROWS             ;  ... what we've got
              MOV       DI,[pNumCols]
              MOV       [WORD DI],NUMCOLS
              MOV       DI,[pNumBytes]
              MOV       [WORD DI],TOTALCLOCKS         ; clocks = bytes!
              MOV       DI,[ppName]
              MOV       [WORD DI],OFFSET LCDName      ; set up string pointer

;--- clear the buffer

              MOV       [WORD PTR pLCDBuff],0         ; set up LCD pointer
              MOV       [WORD PTR pLCDBuff+2],GLCD_SEGMENT

              LES       DI,[pLCDBuff]
              MOV       AL,FILLVALUE
              MOV       CX,TOTALCLOCKS
              REP STOSB

;--- turn on the panel to show blank screen

              MOV       AX,GLCD_BLZERO      ; gate binary zero to data mux
              OR        AL,GLCD_ON          ; enable the output
              MOV       DX,GLCD_CTLS
              OUT       DX,AL               ; turn the display on
              MOV       [LCDCtlBits],AX     ; save for next time

;--- fill the row starting address table
;    this creates the appropriate RAM address for each row
;     ... the top row is wrapped from the bottom due to Frame Sync offset
;    These are =even= addresses, but the LM215 requires data in successive
;     even/odd byte pairs to provide stable data for the panel!

              MOV       [RowStarts],(ROWMODULUS-1)*ROWCLOCKS
              MOV       [RowStarts+2*ROWMODULUS],(ROWMODULUS-1)*ROWCLOCKS

              MOV       DI,(OFFSET RowStarts) + 2
              MOV       CX,ROWMODULUS-1
              MOV       AX,1                          ; row 0 = second entry
              PUSH      DS                            ; set up STOSW segment
              POP       ES

@@1:          MOV       [DI+2*ROWMODULUS],AX          ; second half of table
              STOSW                                   ; first half
              ADD       AX,ROWCLOCKS                  ; offset of next row
              LOOP      @@1

              RET

              ENDP      LCDInit

;-------------
; Create the LCD buffer address given row and column numbers
; Note that we pass parameters in registers rather than on the stack...
;  AX = column number, BX = row number
; Returns DI set to the byte offset within the buffer
;         CL contains shift needed to move the bit right into LSB (0..7)
; You must set up a seg reg and add the base offset to DI to get a pointer

              PROC      LCDMakeAddr

;--- figure bit shift value

              CMP       AX,COLMODULUS       ; column
              SETAE     CL                  ; CL = 1 if in right half
              ADD       CL,CL               ;  ... make it 2
              CMP       BX,ROWMODULUS       ; upper or lower half?
              SETAE     CH                  ; CH = 1 if in lower half
              ADD       CL,CH               ; get total shift amount

              ADD       BX,BX               ; make word table index
              MOV       DI,[RowStarts+BX]   ;  ... to get row start

              DIV       [BYTE ColModulus]   ; get column offset
              MOV       AL,AH               ; remainder = col offset
              XOR       AH,AH               ; always less than 256...
              ADD       AX,AX               ; times 2 for addressing

              ADD       DI,AX               ; set up complete address

              RET

              ENDP      LCDMakeAddr

;-------------
; Plot a point on the LCD panel
; This always uses the buffer pointed to by pNewBuff

              PUBLIC    C LCDSetDot
              PROC      LCDSetDot
              ARG       Row:WORD,Col:WORD,Value:WORD
              USES      ES,DI

              MOV       AX,[Col]
              MOV       BX,[Row]
              CALL      LCDMakeAddr

              LES       AX,[pNewBuff]       ; get buffer base pointer
              ADD       DI,AX               ;  ... update byte pointer

              MOV       BX,0101h            ; set up bit mask
              SHL       BX,CL
              NOT       BL                  ;  BL = AND mask, BH = OR mask

              TEST      [Value],0001h       ; what do we want?
              JNZ       @@2                 ; nonzero means set the bit
              AND       [ES:DI],BL          ;  ... clear it
              AND       [ES:DI+1],BL        ;  ... do the odd byte, too
              JMP       SHORT @@3

@@2:
              OR        [ES:DI],BH          ;  ... set it
              OR        [ES:DI+1],BH        ;  ... do the odd byte, too

@@3:
              RET

              ENDP      LCDSetDot

;-------------
; Fetch a point from the LCD panel
; Always uses the buffer pointed to by pOldBuff
; We return 0 for any addresses outside the current panel
;  ... which greatly simplifies generating dots for The Game of Life!

              PUBLIC    C LCDGetDot
              PROC      LCDGetDot
              ARG       Row:WORD,Col:WORD
              USES      ES,DI

              XOR       AX,AX               ; set up zero return value

              MOV       BX,[Col]            ; verify column
              TEST      BH,80h              ; negative?
              JNZ       @@Done              ;  ... yes
              CMP       BX,NUMCOLS          ; too big?
              JAE       @@Done              ;  ... yes

              MOV       BX,[Row]            ; verify row
              TEST      BH,80h              ; negative?
              JNZ       @@Done              ;  ... yes
              CMP       BX,NUMROWS          ; too big?
              JAE       @@Done              ;  ... yes

              MOV       AX,[Col]            ; set up column number
              CALL      LCDMakeAddr

              LES       AX,[pOldBuff]       ; get buffer base pointer
              ADD       DI,AX               ;  ... update byte pointer

              MOV       AL,[ES:DI]          ;  ... fetch the byte
              SHR       AL,CL               ; skid our bit into LSB
              AND       AX,0001h            ;  ... isolate it
@@Done:
              RET

              ENDP      LCDGetDot

              END
