/* Embedded PC's ISA Bus Chapter 16 -- Ed Nisley                             */
/* ANSI routines for Graphic LCD Interface                                   */
/* Compiled with Borland C++ 3.1, converted to binary with Paradim Locate    */
/* Requires diskette boot sector loader to 1000:0000                         */

#include <conio.h>
#include <dos.h>
#include <stdio.h>
#include <stdlib.h>

#include "8086base.h"
#include "firmdev.h"
#include "utility.h"
#include "console.h"

#include "LCDPlot.h"
#include "LCDChars.h"
#include "LCDAnsi.h"

/*---------------------------------------------------------------------------*/
/* Useful constants                                                          */

#define DELAY_LONG   1000000L
#define DELAY_SHORT   250000L

/*---------------------------------------------------------------------------*/
/* Useful routines                                                           */

int GetInt(char *pMsg);
void GetCoord(char *pMsg, int *pRow, int *pCol);

int GetInt(char *pMsg) {

char Buffer[80];
int RetVal;
int Counter;

   do {
      printf("%s",pMsg);
      gets(Buffer);
      Counter = sscanf(Buffer," %u ",&RetVal);
   } while (1 != Counter);

   return RetVal;

}


void GetCoord(char *pMsg, int *pRow, int *pCol) {

char Buffer[80];
int Counter;

   do {
      printf("%s",pMsg);
      gets(Buffer);
      Counter = sscanf(Buffer," %u %u ",pRow,pCol);
   } while (2 != Counter);


}


/*---------------------------------------------------------------------------*/
/* The Main Event...                                                         */

void  main(void)
{

int Row,Col;
int Counter;
int Height;
int Width;
int CharSkew;
int Option;
char Buffer[80];

   outpw(CTLS_ADDR,0);                 /* allow GLCD RAM buffer access       */

   _putch(FORMFEED);
   puts("Graphic LCD Interface: ANSI Cursor Controls");
   puts("Embedded PC's ISA Bus Chapter 16 -- Ed Nisley");

/*--- fire up the hardware                                                   */

   puts("Initializing LCD hardware...");
   LCDInit();
   printf("  %s is now active \n"
          "  %u dot rows, %u dot cols, blinking %s, %u buffer bytes\n",
           pLCDName,NumDotRows,NumDotCols,
           BlinkAllowed ? "allowed" : "disabled",
           NumDotBytes);

   LCDInitFont();
   LCDAnsiInit();

   CharSkew = 0;
   while (1) {

      puts("\nCharacter LCD interface checkout routines:");
      puts(" 0 - Set ANSI decoder trace mode");
      puts(" 1 - Write character test pattern");
      puts(" 2 - Clear panel");
      puts(" 3 - Scroll upward");
      puts(" 4 - Scroll block leftward");
      puts(" 5 - Set cursor position & write string");
      puts(" 6 - ANSI cursor control test sequences");
      printf("Select option: ");

      Option = getchar();
      puts("    (press any key to exit test loops)\n");

      switch (Option) {
      case '0' :
         Row = GetInt("Trace mode: ");
         LCDAnsiTrace(Row);
         break;
      case '1' :
         puts("Filling panel with 00..FF character sequence");
         Counter = CharSkew++;
         for (Row=0; Row<NumCharRows; ++Row) {
            for (Col=0; Col<NumCharCols; ++Col) {
               LCDSetBlink((Row+Col) & 0x0001);
               LCDWriteGlyph(Counter,Row*FontHeight,Col*FontWidth);
               ByteToLEDs(Row);
               Counter = (++Counter) % 256;
            }
         }
         LCDSetBlink(0);
         break;
      case '2' :
         puts("Clearing panel");
         LCDClear();
         break;
      case '3' :
         Row = GetInt("Enter number of =dot= rows to scroll: ");
         printf("  Scrolling up by %u dot rows\n",Row);
         LCDScrollUp(Row);
         break;
      case '4' :
         GetCoord("Enter row and column in dot coordinates: ",&Row,&Col);
         GetCoord("  height & width of area to scroll: ",&Height,&Width);
         LCDScrollLeft(Row,Col,Height,Width);
         break;
      case '5' :
         GetCoord("Enter row and column in dot coordinates: ",&Row,&Col);
         printf("  text string: ");
         gets(Buffer);
         LCDSetDotCursor(Row,Col);
         LCDSendString(Buffer);
         break;
      case '6' :
         LCDAnsiSendString(ANSI_CLEAR         "Clear and home");
         BIOSDelay(DELAY_LONG);
         LCDAnsiSendString(ANSI_SETCUR(10,10) "x <- row 10, column 10");
         BIOSDelay(DELAY_SHORT);
         LCDAnsiSendString(ANSI_CURDOWN(2)    "Down to row 12");
         BIOSDelay(DELAY_SHORT);
         LCDAnsiSendString(ANSI_CURLEFT(44)   "Left to col 2");
         BIOSDelay(DELAY_SHORT);
         LCDAnsiSendString(ANSI_CURUP(1)      "The rest of this line will be "
                                              "erased [................]");
         BIOSDelay(DELAY_LONG);
         LCDAnsiSendString(ANSI_CURLEFT(99) ANSI_CURRIGHT(50) ANSI_CLREOL);
         BIOSDelay(DELAY_LONG);

         LCDAnsiSendString(ANSI_CURDOWN(3) ANSI_CURLEFT(30));
         LCDAnsiSendString("Cursor saved here ->" ANSI_SAVECUR);
         BIOSDelay(DELAY_SHORT);
         LCDAnsiSendString("moved below...");
         BIOSDelay(DELAY_SHORT);
         LCDAnsiSendString(ANSI_SETCUR(20,30) "x <- Moved to here (20,30)");
         BIOSDelay(DELAY_LONG);
         LCDAnsiSendString(ANSI_RSTCUR "and then restored");
         BIOSDelay(DELAY_LONG);

         if (NumCharRows > 25) {
            LCDAnsiSendString(ANSI_SETCUR(49,63) "No scroll from the");
            LCDAnsiSendString(ANSI_SETCUR(50,67) "last char -> x");
         }
         else {
            LCDAnsiSendString(ANSI_SETCUR(24,63) "No scroll from the");
            LCDAnsiSendString(ANSI_SETCUR(25,67) "last char -> x");
         }
         BIOSDelay(DELAY_SHORT);
         LCDAnsiSendString(ANSI_SETCUR(18,51));
         LCDAnsiSendString("This line is split from row 18"
                           "with H wrap disabled");
         LCDAnsiSendString(ANSI_SETMODE(7));
         LCDAnsiSendString(ANSI_SETCUR(22,49));
         LCDAnsiSendString("This line is wrapped from col 80"
                           "with H wrap enabled ...........................");
         LCDAnsiSendString(ANSI_RSTMODE(7));
         LCDAnsiSendString(ANSI_SETCUR(23,51));
         LCDAnsiSendString("This line is split from row 22"
                           "with H wrap enabled again");
         BIOSDelay(DELAY_LONG);

         LCDAnsiSendString(ANSI_SETCUR(3,40));
         if (BlinkAllowed) {
            LCDAnsiSendString(ANSI_SETBLINK "This is blinking");
            LCDAnsiSendString(ANSI_CLRATTR);
         }
         else {
            LCDAnsiSendString("This panel does not blink");
         }
         BIOSDelay(DELAY_LONG);

         LCDAnsiSendString(ANSI_SETCUR(15,1));
         LCDAnsiSendString("1234567890123456789012345678901234567890"
                           "1234567890123456789012345678901234567890");
         LCDAnsiSendString("tab\tstops\tare\ton\tfour-character\t"
                           "intervals\tacross\tthe\tpanel");
         BIOSDelay(DELAY_LONG);

         LCDAnsiSendString(ANSI_HOME  "Just.....");
         LCDAnsiSendString(ANSI_SETCUR(1,20)  "and done!");
         break;
      default :
         puts("*** Choose one of the menu options\n");
      }

   }

}
