/*
//    Paradigm LOCATE Console I/O Driver
//
//    Function
//    ========
//    This module contains a generic console device driver for the
//    Paradigm stream I/O interface containing the following functions.
//
//       void     InitConsole(void) ;
//       int      _getch(void) ;
//       int      _putch(int) ;
//
//    Modified to use BIOS serial I/O functions... Dec 93 Ed Nisley
//    You will need the Paradigm LOCATE\BCPP31\HELPERS files to recompile
//     any programs using this file.
*/

#include <dos.h>
#define COMPORT 1                      /* 1 = COM1, 2 = COM2, etc...         */

#include "typedefs.h"               /* General purpose type declarations */
#include "dosemu.h"                 /* Int 21h function/data prototypes */


/*
//    Function:  _getch
//
//    This function returns the next character received from a serial
//    device.
*/

int   _getch(void)
{
union REGS regs;

   do {
      regs.h.ah = 3;                   /* status check                       */
      regs.x.dx = COMPORT - 1;
      int86(0x14,&regs,&regs);
   } while (!(regs.h.ah & 0x01));      /* wait for data ready...             */

   regs.h.ah = 2;                      /* fetch character                    */
   regs.x.dx = COMPORT - 1;
   int86(0x14,&regs,&regs);            /* we ignore errors here, sigh.       */

   return 0x7F & regs.h.al;
}


/*
//    Function:  _putch
//
//    This function must be written to copy the supplied character to
//    to a serial device.
*/

int   _putch(
         int      ch                /* Character to output */
      )
{
union REGS regs;

   do {
      regs.h.ah = 3;
      regs.x.dx = COMPORT - 1;         /* here 0 = COM1, etc                 */
      int86(0x14,&regs,&regs);
   } while (!(regs.h.ah & 0x20));      /* wait for trans holding reg empty   */

   regs.h.ah = 1;                      /* send data                          */
   regs.h.al = ch;
   regs.x.dx = COMPORT - 1;
   int86(0x14,&regs,&regs);            /* ignore errors                      */

   return   ch ;
}


/*
//    Function:   InitConsole
//
//    This function is responsible for initializing the console device
//    used by the stream I/O package.
*/

void     InitConsole(void)
{
union REGS regs;

   regs.h.ah = 0;                      /* initialization                     */
   regs.h.al = 0xE3;                   /* 9600, 1 stop, no parity, 8 data    */
   regs.x.dx = COMPORT - 1;            /* here 0 = COM1, etc                 */
   int86(0x14,&regs,&regs);            /* we ignore errors here, too         */

}
