/* ANSI.H                                                                    */
/* Useful stuff for ANSI control sequences                                   */

#define ESC_LEAD   "\x01b["            /* ESC [ lead chars for all sequences */

/*---------------------------------------------------------------------------*/
/* Attribute names for ESC[...m sequences                                    */
/* Not all of these will work on an LCD panel...                             */

#define ATTR_OFF     0
#define ATTR_BOLD    1
#define ATTR_FAINT   2
#define ATTR_BLINK   5
#define ATTR_REVERSE 7

/*--- colors, must be added to 30 for FG and 40 for BG sequences             */

#define ATTR_BLACK   0
#define ATTR_RED     1
#define ATTR_GREEN   2
#define ATTR_YELLOW  3
#define ATTR_BLUE    4
#define ATTR_MAGENTA 5
#define ATTR_CYAN    6
#define ATTR_WHITE   7

#define ATTR_NUMCOLORS 8


/*---------------------------------------------------------------------------*/
/* Macros to generate some useful sequences                                  */
/* These must be used where string pasting works...                          */

#define ANSI_CLEAR          ESC_LEAD "2J"          /* clear screen           */
#define ANSI_CLREOL         ESC_LEAD "K"           /* clear to end of line   */
#define ANSI_SETCUR(r,c)    ESC_LEAD #r ";" #c "f" /* set cursor position    */
#define ANSI_SETMODE(x)     ESC_LEAD "=" #x "h"    /* set screen mode        */
#define ANSI_RSTMODE(x)     ESC_LEAD "=" #x "l"    /* reset screen mode      */
#define ANSI_SETBLINK       ESC_LEAD "7m"          /* enable blink           */
#define ANSI_CLRATTR        ESC_LEAD "0m"          /* clear attrs            */
#define ANSI_SETFG(x)       ESC_LEAD "3" #x "m"    /* set foreground color   */
#define ANSI_SETBG(x)       ESC_LEAD "4" #x "m"    /* set background color   */
#define ANSI_SAVECUR        ESC_LEAD "s"           /* save cursor position   */
#define ANSI_RSTCUR         ESC_LEAD "u"           /* save cursor position   */
#define ANSI_CURUP(x)       ESC_LEAD #x "A"        /* move up                */
#define ANSI_CURDOWN(x)     ESC_LEAD #x "B"        /* move down              */
#define ANSI_CURRIGHT(x)    ESC_LEAD #x "C"        /* move right             */
#define ANSI_CURLEFT(x)     ESC_LEAD #x "D"        /* move left              */
#define ANSI_HOME           ESC_LEAD "H"           /* move to upper left     */


/*---------------------------------------------------------------------------*/
/* Prototypes                                                                */

void LCDAnsiInit(void);
void LCDAnsiDriver(int NewChar);
void LCDAnsiTrace(int Trace);

void LCDAnsiSendString(char *pString);


