/* Utility routines for the Firmware Development Board                       */

#include <dos.h>


#include "8086base.h"
#include "firmdev.h"
#include "utility.h"

/*---------------------------------------------------------------------------*/
/* Convert byte to 7-segment display and write to port                       */
/* The decimal points aren't used...                                         */

void ByteToLEDs(BYTE Value) {

static BYTE LEDSegs[16] = {0x7E,0x30,0x6D,0x79,0x33,0x5B,0x5F,0x70,
                           0x7F,0x7B,0x77,0x1F,0x4E,0x3D,0x4F,0x47};

WORD LEDValue;

   Value &= 0x00FF;                    /* ensure only a byte in here...      */

   LEDValue = (LEDSegs[Value >> 4] << 8) | LEDSegs[Value & 0x000F];

   outpw(LED_ADDR,~LEDValue);          /* display it on the lEDs             */

}



/*---------------------------------------------------------------------------*/
/* Load Firmware Development Board 8254 counter channel                      */
/* Uses constant for board address, which seems reasonable...                */

void Load8254(WORD Channel, WORD Ctl, WORD Reload) {

WORD CtlValue;

   outpw(I8254_BASE+6,(Channel << 6) | Ctl | 0x30);

   outpw(I8254_BASE+2*Channel,Reload);

   outpw(I8254_BASE+2*Channel,Reload >> 8);

}



/*---------------------------------------------------------------------------*/
/* Delay for a specified number of microseconds                              */
/*  ... actually accurate to about a millisecond, more or less               */
/* Uses the BIOS delay function to get accurate timing...                    */
/* Returns 0 if OK, 1 if error                                               */

int  BIOSDelay(unsigned long MicroSecs) {

union REGS regs;

   regs.x.ax = 0x8600;                 /* Set Wait Interval function         */
   regs.x.cx = (WORD)(MicroSecs >> 16);            /* high part of delay     */
   regs.x.dx = (WORD)MicroSecs;                    /*  ... low part          */

   int86(0x15,&regs,&regs);            /* do the deed                        */

   return regs.x.cflag;                /* carry set on error                 */

}

