*
* MICRO-C Runtime library for: 8086 Family (standalone)
* SMALL model modified for BIOS Extension use
*  7 Aug 93 Ed Nisley
*
* Copyright 1988,1992 Dave Dunfield
* All rights reserved.
*
*--- Useful constants
*
STK_INT       EQU       $60                 interrupt vector for RAM segment
STK_TOP       EQU       $7FFE               one word below boundary
*
LED_ADDR      EQU       $031E               LED output port
STAT_ADDR     EQU       $031C               status input port
PUSHBUTTON    EQU       $0200               pushbutton bit location
*
*-------------
* BIOS Extension header
*
              ORG       $0100               standard COM offset
*
              DB        $55                 signature
              DB        $AA                  ... second byte
              DB        16                  code + init data (512 byte units)
              JMP       <BootEntry          jump around checksum
              DB        $00                  ... which goes here
*
*--- if pushbutton is down, exit without doing much
*
BootEntry     EQU       *
              MOV       DX,#STAT_ADDR
              IN        AX,DX
              TEST      AX,#PUSHBUTTON
              JNZ       <Continue           nonzero means not pushed
*
              MOV       AX,#~$0101          show -- to track our path
              MOV       DX,#LED_ADDR
              OUT       DX,AX
*
              RETF                          ; return to normal BIOS boot
*
*--- pushbutton is up, so it is OK to continue
*    adjust CS so our offsets are correct
*
Continue      EQU       *
              MOV       AX,#~$0100          show single - here
              MOV       DX,#LED_ADDR
              OUT       DX,AX
*
              MOV       AX,CS
              SUB       AX,#$0010           add 0100 to offsets!
              PUSH      AX                  simulate a FAR CALL
              MOV       AX,#LdRegs
              PUSH      AX
              RETF                          ;  ... to load the new values
LdRegs        EQU       *
*
*--- locate the data segment at the top 32K of contiguous RAM
*    the BIOS RAM size value at 40:13 has units of 1K bytes
*
              MOV       AX,#$057E           show r0 on LEDs to mark entry
              CALL      ShowBits
*
              PUSH      DS                  save BIOS seg regs
              PUSH      ES                   ... on the BIOS stack
*
              MOV       AX,#$0040           fetch memory size
              MOV       ES,AX
              MOV       BX,#$0013
              SUB       ES:[BX],#>$0020     reserve 32K (1K byte units)
              MOV       AX,ES:[BX]          convert RAM size to seg reg
              MOV       CL,#6                 (10 bits left, 4 bits right)
              SHL       AX,CL
              MOV       ES,AX
*
*--- copy the initialized data to the data segment
*    the length is just the offset of ?temp...
*     ... as long as ?temp is the first uninitialized variable!
*
              MOV       AX,#$0530           ; r1
              CALL      ShowBits
*
              MOV       AX,CS               aim at this segment
              MOV       DS,AX
              MOV       CX,#?temp           pick up offset as length
              MOV       SI,#?initdata       start of init data in code seg
              MOV       DI,#0               copied to start of data seg
              JCXZ      ?nocopy             skip if zero (or 64 K) bytes long
              REP
              MOVSB                         from DS:SI to ES:DI
?nocopy       EQU       *
*
*--- clear the uninitialized data while we have the pointers at hand
*    length is the difference between ?heap and ?temp
*     ... as long as ?heap is just beyond the last variable
*
              MOV       AX,#$056D           r2
              CALL      ShowBits
*
              MOV       CX,#?heap
              MOV       AX,#?temp           work around for phase error bug
              SUB       CX,AX
              MOV       AL,#0               get a zero (or a test value!)
              REP
              STOSB
*
*--- aim DS at the new data segment
*
              MOV       AX,ES
              MOV       DS,AX
*
*--- save the BIOS SS:SP registers
*     ... begin using our stack in the new data segment
*    then save the pointer in an interrupt vector for later use
*
              MOV       ?BIOS_SP,SP
              MOV       ?BIOS_SS,SS
*
              MOV       SS,AX               load stack pointer
              MOV       SP,#STK_TOP
*
              XOR       BX,BX               set interrupt vector
              MOV       ES,BX
              MOV       BX,#STK_INT
              ADD       BX,BX
              ADD       BX,BX
              MOV       ES:[BX],SP
              MOV       ES:2[BX],SS
              MOV       ES,AX               restore ES
*
*--- now, for the main event...
*
              MOV       AX,#$0579           r3
              CALL      ShowBits
*
              MOV       <?heap,#0           set up empty heap
              CALL      main                Execute main program
*
*--- restore regs and return
*
              MOV       SS,?BIOS_SS         aim back at BIOS stack
              MOV       SP,?BIOS_SP
*
              POP       ES                  from BIOS stack
              POP       DS
*
              MOV       AX,#$0505           show rr to indicate the end
              CALL      ShowBits
*
              RETF                          ; and continue with boot
*
*-------------
* Write AX to the LED digits
*
ShowBits      EQU       *
              PUSH      DX
              MOV       DX,#LED_ADDR
              NOT       AX                  ; make 1 bits light LEDs
              OUT       DX,AX
              POP       DX
              RET
*
*-------------
* Standard Micro-C startup code begins here
*
* Compare library routines
?eq     CMP     AX,CX           CX == AX
        JZ      TRUE
        XOR     AX,AX
        RET
?ne     CMP     AX,CX           CX != AX
        JNZ     TRUE
        XOR     AX,AX
        RET
?lt     CMP     AX,CX           CX < AX
        JG      TRUE
        XOR     AX,AX
        RET
?le     CMP     AX,CX           CX <= AX
        JGE     TRUE
        XOR     AX,AX
        RET
?gt     CMP     AX,CX           CX > AX
        JL      TRUE
        XOR     AX,AX
        RET
?ge     CMP     AX,CX           CX >= AX
        JLE     TRUE
        XOR     AX,AX
        RET
?ult    CMP     AX,CX           CX < AX (unsigned)
        JA      TRUE
        XOR     AX,AX
        RET
?ule    CMP     AX,CX           CX <= AX (unsigned)
        JAE     TRUE
        XOR     AX,AX
        RET
?ugt    CMP     AX,CX           CX > AX (unsigned)
        JB      TRUE
        XOR     AX,AX
        RET
?uge    CMP     AX,CX           CX >= AX (unsigned)
        JBE     TRUE
FALSE   XOR     AX,AX
        RET
* logically negate (ax)
?not    AND     AX,AX           Test for non-zero
        JNZ     FALSE           Not zero, invert
TRUE    MOV     AX,#1           Create a true value
        AND     AX,AX           Set condition code
nargs   RET
* Handle switch statement
?switch MOV     CX,CS:[BX]      Get label value
        JCXZ    SWEND           End of table - Special case
        ADD     BX,#4           Skip to next
        CMP     AX,CS:-2[BX]    Does it match?
        JNZ     ?switch         No, keep looking
        JMP     CX              Execute case code
SWEND   JMP     CS:2[BX]        Execute default code
*
$SE:1
*-------------
* Segment 1 - Initialized data
* The ?initdata label marks the =end= of the code segment
*  ... the ORG resets the address to 0 in the data segment
?initdata     EQU       *
        ORG     $0000           Begin initialized data here
*
$DD:?temp 2
$DD:?BIOS_SP 2
$DD:?BIOS_SS 2
