@ECHO OFF
REM ----------------------
REM verify that we were set up properly
IF "%mcsetup%" == "yes" GOTO good
ECHO *** Use MCSETUP before trying MCCOMP...
SET beeps=1
GOTO done
:good
CLS
ECHO Micro-C 8086 compile: %1
REM ----------------------
REM set up a few defaults
IF "%mclib%" == "" SET mclib=%mcdir%
IF "%mcinclude%" == "" SET mclib=%mcdir%
IF "%mcromend%" == "" SET mcromend=1fff
SET model=%mcmodel%
IF "%model%" == "" SET model=tiny
SET discard=yes
SET quiet=-q
SET beeps=1
REM ----------------------
REM extract command line parameters
IF "%1" == "" goto help
SET target=%1
SHIFT
:parm1
IF "%1" == "keep" SET discard=no
IF "%1" == "keep" GOTO parm2
IF NOT "%1" == "" SET model=%1
:parm2
SHIFT
IF NOT "%1" == "" GOTO parm1
:parm9
GOTO run
REM ----------------------
:help
ECHO ---
ECHO Syntax is %0 filename [memorymodel] [keep]
ECHO  Don't include the extension on the filename
ECHO  memorymodel:  overrides MCMODEL environment variable
ECHO  keep:  preserves temporary files in %mctmp%
ECHO  (lower case for everything...)
ECHO ---
ECHO These environment variables are useful:
ECHO  MCDIR      subdirectory containing executable files       %mcdir%
ECHO  MCTMP      subdirectory for temporary files               %MCTMP%
ECHO  MCLIB      subdirectory for 8086 library files            %MCLIB%
ECHO  MCINCLUDE  subdirectory for 8086 header files             %MCINCLUDE%
ECHO  MCCUSTOM   subdirectory for customized files              %MCCUSTOM%
ECHO  MCMODEL    memory model:  tiny/small/biosext              %MCMODEL%
ECHO  MCMODEXT   suffix for memory model filename               %MCMODEXT%
ECHO ---
goto done
REM ----------------------
REM Do the deed...
:run
ECHO --- clean up old files
IF exist %mctmp%.\%target%.CP  del %mctmp%.\%target%.CP
IF exist %mctmp%.\%target%.AS* del %mctmp%.\%target%.AS*
IF exist %mctmp%.\%target%.OPT del %mctmp%.\%target%.OPT
IF exist          %target%.LST del          %target%.LST
IF exist %target%.HEX del %target%.HEX
ECHO --- C preprocessor
MCP %target%.c %mctmp%.\%target%.CP l=%mcinclude% %quiet% -c
IF ERRORLEVEL 1 GOTO errors
ECHO --- C compiler
MCC86 %mctmp%.\%target%.CP %mctmp%.\%target%.AS1 %quiet%
IF ERRORLEVEL 1 GOTO errors
IF %discard% == yes DEL %mctmp%.\%target%.CP
ECHO --- ASM preprocessor
MACRO %mccustom%\macros.inc %mctmp%.\%target%.AS1 > %mctmp%.\%target%.AS2
ECHO --- optimizer
MCO86 %mctmp%.\%target%.AS2 %mctmp%.\%target%.AS3 %quiet% -d > %mctmp%.\%target%.OPT
rem copy %mctmp%.\%target%.AS2 %mctmp%.\%target%.AS3
IF ERRORLEVEL 1 GOTO errors
IF %discard% == yes DEL %mctmp%.\%target%.AS1
IF %discard% == yes DEL %mctmp%.\%target%.AS2
IF %discard% == yes DEL %mctmp%.\%target%.OPT
ECHO --- source linker [using library %model%%mcmodext%.lib]
SLINK %mctmp%.\%target%.AS3 %mctmp%.\%target%.ASM %quiet% i=%model%%mcmodext%.lib t=%mctmp% l=%mclib%
IF ERRORLEVEL 1 GOTO errors
IF %discard% == yes DEL %mctmp%.\%target%.AS3
ECHO --- assembler
ASM86 %mctmp%.\%target%.ASM -fis l=%target%.lst c=%mctmp%.\%target%.hxx
IF ERRORLEVEL 1 GOTO errors
IF %discard% == yes DEL %mctmp%.\%target%.ASM
rem ECHO --- checksum
REM the checksum offset is relative to first HEX address ---vvvv)
rem *** this requires a matching change in 8086RLPx.ASM
rem *** disabled until we need it...
rem HEXFMT %mctmp%.\%target%.hxx -i s=$8000 c=$0000,$%mcromend%,$0002 w=%target%.hex
rem IF ERRORLEVEL 1 GOTO errors
rem IF %discard% == yes DEL %mctmp%.\%target%.HXX
copy %mctmp%.\%target%.hxx %target%.hex
REM ----------------------
REM Convert HEX file to BIN (and copy to diskette if needed)
REM For BIOSEXT model programs, we resequence the HEX file addresses
IF %mcmodel% == biosext hexseq %target%.hex
HEXFMT %target%.hex -b s=$4000 w=%target%.bin
IF NOT "%download%" == "yes" GOTO done
DEL A:*.bin
COPY %target%.bin a:
ECHO --- Done!
GOTO done
:errors
ECHO *** Got problems, mon...
SET beeps=1 2
:done
for %%f IN (%beeps%) do echo 
REM ----------------------
REM Clean up the environment
SET beeps=
SET discard=
SET model=
SET quiet=
SET target=
