* Replacement for SERIO.ASM to use serial port
* Embedded PC's ISA Bus -- Ed Nisley
*
*-------------------
* Set up the serial port
*  serinit(bitrate,COMport)
* Assumes 8N1 format
*
serinit       PUSH      BP                  save stack frame pointer
              MOV       BP,SP               Get pointer to params on stack
              MOV       BX,DS               set up ES for SCAS
              MOV       ES,BX
              MOV       AX,6[BP]            fetch bit rate (stacked first)
              MOV       CX,#8               Set table length
              MOV       DI,#?bittab         Set table base
              REPNE
              SCASW                         Search for rate
              MOV       AL,CL
              NEG       AL                  Convert count to table index
              ADD       AL,#7                (if not found, use last entry)
              MOV       CL,#5               Move bits 2:0 to 7:5
              ROL       AL,CL
              AND       AL,#$E0             clean out low bits
              OR        AL,#$03             Set no parity, 1 stop, 8 data
              MOV       AH,#0               BIOS function number
              MOV       DX,4[BP]            Get COM number (stacked last)
              DEC       DX                  Make COM1 = 0
              MOV       ?COMport,DX         save for later
              INT       $14                 Do the BIOS initialization
* set up modem control outputs to ensure good communications
              MOV       BX,?COMport         get port index
              ADD       BX,BX                ... convert to word index
              MOV       AX,#$0040           aim ES at BIOS data area
              MOV       ES,AX
              MOV       DX,ES:[BX]          fetch port address
              ADD       DX,#$0004           aim at modem controls
              IN        AL,DX               fetch current bits
              OR        AL,#$03             force RTS & DTR high (inverted)
              OUT       DX,AL
              POP       BP                  restore frame pointer
              RET
*
*-------------
* Table of BIOS-acceptable bit rates
* The entry index is the BIOS bit rate code value
*
?bittab       DW        110                 0
              DW        150                 1
              DW        300                 2
              DW        600                 3
              DW        1200                4
              DW        2400                5
              DW        4800                6
              DW        9600                7
*
*
* Write a string to the serial port
putstr  MOV     BX,SP           Address stack
        MOV     SI,SS:2[BX]     Get string
?putstr MOV     AL,[SI]         Get char
        INC     SI              Skip to next
        AND     AL,AL           End of string?
        JZ      ?1              Yes, exit
        CALL    ?putch          Write it
        JMP     <?putstr        Do it all
*
* Write char with NEWLINE<>CR translation
*
putch   MOV     BX,SP           Address stack
        MOV     AX,SS:2[BX]     Get character
?putch  CALL    ?putchr         Write value
        CMP     AL,#$0A         Newline?
        JNZ     ?1              No, skip it
        MOV     AL,#$0D         Add carriage return
        JMP     <?putchr        Write it out
*
* Write char with no translations
*
putchr  MOV     BX,SP           Address stack
        MOV     AX,SS:2[BX]     Get character
* Write through IBM/PC BIOS call
?putchr MOV     DX,?COMport     use our port
        MOV     AH,#$01         Write char to port
        INT     $14             Call BIOS
?1      RET
*
* Hang until a character arrives
* Return value is raw character in AL
*
waitchr MOV     AH,#$03         Status request
        MOV     DX,?COMport     use our port
        INT     $14             Ask BIOS
        TEST    AH,#$01         Check data ready
        JZ      waitchr         Char is ready
        MOV     AH,#$02         Get character
        MOV     DX,?COMport     use our port
        INT     $14             Ask BIOS
        XOR     AH,AH           clear high byte
        RET
*
* Test for character (No translation)
*
chkchr  MOV     AH,#$03         Status request
        MOV     DX,?COMport     use our port
        INT     $14             Ask BIOS
        TEST    AH,#$01         Check data ready
        MOV     AX,-1           assume no char
        JZ      ?ccr            correct, bail out
        MOV     AH,#$02         Get char
        MOV     DX,?COMport     use our port
        INT     $14             Ask BIOS
        XOR     AH,AH           discard flags
?ccr    RET
*
* Receive char with no translations
*
getchr  CALL    waitchr         Wait for character and get it
        RET
*
* Test for character (with CR<>NEWLINE translation)
*
chkch   MOV     AH,#$03         Status request
        MOV     DX,?COMport     use our port
        INT     $14             Ask BIOS
        TEST    AH,#$01         Check data ready
        MOV     AX,#0           assume nothing
        JZ      ?cch            correct
        MOV     AH,#$02         Get character
        MOV     DX,?COMport     use our port
        INT     $14             Ask BIOS
        XOR     AH,AH           discard flags
?cch    RET
*
* Receive char with CR<>NEWLINE translation
*
getch   CALL    waitchr         Wait for char and get it
        XOR     AH,AH           Zero HIGH
        CMP     AL,#$0D         Carriage return
        JNZ     ?1              No, its OK
        MOV     AL,#$0A         Convert to NEWLINE
        RET
*
* Receive a string: getstr(buffer, size)
*
getstr  MOV     BX,SP           Address stack
        MOV     SI,SS:4[BX]     Get buffer address
        XOR     CX,CX           Zero CX
?2      CALL    getch           Get character
        CMP     AL,#$08         Backspace?
        JZ      ?3              Yes, handle it
        CMP     AL,#$7F         Delete?
        JZ      ?3              Yes, handle it
        CMP     AL,#$0A         End of line?
        JZ      ?4              Yes, handle it
        CMP     CX,SS:2[BX]     Are we at end?
        JAE     ?2              Yes, don't accept
        MOV     [SI],AL         Write it
        INC     SI              Advance pointer
        INC     CX              Advance length
        CALL    ?putch          Write it out
        JMP     <?2             And proceed
* Delete character from buffer
?3      AND     CX,CX           At beginning?
        JZ      ?2              Yes, ignore
        MOV     AL,#$08         Backup
        CALL    ?putch          Output
        MOV     AL,#' '         Space over
        CALL    ?putch          Output
        MOV     AL,#$08         Backup
        CALL    ?putch          Output
        DEC     SI              Backup pointer
        DEC     CX              Reduce length
        JMP     <?2             And proceed
* Newline, terminate entry
?4      CALL    ?putch          Echo it
        MOV     >[SI],#0        Zero terminate
        MOV     AX,CX           Return length
        RET
$DD:?COMport 2
